'''This module implements additional complex configuration entries.

The following types of entries are provided:
FileEntry (type=='file')
FontEntry (type=='font')
ColourEntry (type=='colour')
'''

import gtk
import gtk.gdk

import finalconfig.entry


class FileEntry (finalconfig.entry.Entry):
    '''File configuration entry.
    
    This entry has the following elements:
    title
    hint (optional)
    value
    '''
    
    def __init__ (self, id, directory):
        finalconfig.entry.Entry.__init__ (self)
        self.title = ''
        self.id = id
        self.state = ''
        self.default = ''
        self.tooltip = None
        self.widget = None
        self.label = None
        self.directory = directory
        self.filter_ = []
        
    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:        
            self.widget = gtk.FileChooserButton (self.title)
            self.widget.set_filename (self.state)
            for f in self.filter_:
                self.widget.add_filter (f)
            self.label = gtk.Label (self.title)
            if self.directory:
                self.widget.set_action (gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            else:
                self.widget.set_action (gtk.FILE_CHOOSER_ACTION_OPEN)
            self.label.set_alignment (0.0, 0.5)            
            if self.tooltip is not None:
                self.widget.set_tooltip_text (self.tooltip)
                self.label.set_tooltip_text (self.tooltip)            
        return (self.label, self.widget)

    def set_config (self, config):
        '''Read config for this entry from config map.'''
        self.state = config[self.id]
        if self.widget is not None:
            self.widget.set_filename (self.state)

    def get_config (self, config):
        '''Store config for this entry into config map.'''
        if self.widget is not None:
            self.state = self.widget.get_filename()
            if self.state is None:
                self.state = ''
        config[self.id] = self.state    

    def set_default (self, text):
        '''Set the default value for this entry.'''
        self.default = text
        
    def use_default (self):
        '''Set entry to default value.'''
        self.state = self.default
        if self.widget is not None:
            self.widget.set_filename (self.state)

    def set_title (self, text):
        '''Set title (i.e. the label).'''
        self.title = text
        
    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text
    
    def add_filter (self, filter_):
        '''Add a file filter to this entry.'''
        if self.widget:
            self.widget.add_filter (filter_)
        else:
            self.filter_.append (filter_)


class FontEntry (finalconfig.entry.Entry):
    '''Font selection entry.
    
    This entry has the following elements:
    title
    hint (optional)
    value
    '''
    
    def __init__ (self, id):
        finalconfig.entry.Entry.__init__ (self)
        self.title = ''
        self.id = id
        self.state = ''
        self.default = ''
        self.tooltip = None
        self.widget = None
        self.label = None
        
    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:        
            self.widget = gtk.FontButton (self.title)
            self.widget.set_font_name (self.state)
            self.label = gtk.Label (self.title)
            self.label.set_alignment (0.0, 0.5)            
            if self.tooltip is not None:
                self.widget.set_tooltip_text (self.tooltip)
                self.label.set_tooltip_text (self.tooltip)            
        return (self.label, self.widget)

    def set_config (self, config):
        '''Read config for this entry from config map.'''
        self.state = config[self.id]
        if self.widget is not None:
            self.widget.set_font_name (self.state)

    def get_config (self, config):
        '''Store config for this entry into config map.'''
        if self.widget is not None:
            self.state = self.widget.get_font_name()
            if self.state is None:
                self.state = ''
        config[self.id] = self.state    

    def set_default (self, text):
        '''Set the default value for this entry.'''
        self.default = text
        
    def use_default (self):
        '''Set entry to default value.'''
        self.state = self.default
        if self.widget is not None:
            self.widget.set_font_name (self.state)

    def set_title (self, text):
        '''Set title (i.e. the label).'''
        self.title = text
        
    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text


class ColourEntry (finalconfig.entry.Entry):
    '''Colour selection entry.
    
    This entry has the following elements:
    title
    hint (optional)
    value
    '''
    
    def __init__ (self, id, alpha):
        finalconfig.entry.Entry.__init__ (self)
        self.title = ''
        self.id = id
        self.state = '#000'
        self.default = ''
        self.tooltip = None
        self.widget = None
        self.label = None
        self.alpha = alpha
        
    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:        
            self.widget = gtk.ColorButton()
            self.widget.set_title (self.title)
            if self.alpha:
                self.widget.set_use_alpha (True)
            self._set_widget_value (self.state)
            self.label = gtk.Label (self.title)
            self.label.set_alignment (0.0, 0.5)            
            if self.tooltip is not None:
                self.widget.set_tooltip_text (self.tooltip)
                self.label.set_tooltip_text (self.tooltip)            
        return (self.label, self.widget)

    def set_config (self, config):
        '''Read config for this entry from config map.'''
        self.state = config[self.id]
        if self.state is '':
            if self.alpha:
                self.state = '#000 65535'
            else:
                self.state = '#000'
        if self.widget is not None:
            self._set_widget_value (self.state)

    def get_config (self, config):
        '''Store config for this entry into config map.'''
        if self.widget is not None:
            self.state = self._get_widget_value()
        config[self.id] = self.state    

    def set_default (self, text):
        '''Set the default value for this entry.'''
        self.default = text
        
    def use_default (self):
        '''Set entry to default value.'''
        self.state = self.default
        if self.widget is not None:
            self._set_widget_value (self.state)

    def set_title (self, text):
        '''Set title (i.e. the label).'''
        self.title = text
        
    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text

    def _get_widget_value (self):
        colour = str (self.widget.get_color())
        if not self.alpha:
            return colour
        return colour + ' ' + str (self.widget.get_alpha())
        
    def _set_widget_value (self, value):
        (colour, sep, alpha) = value.partition (' ')
        self.widget.set_color (gtk.gdk.color_parse (colour))
        if self.alpha:
            if len (alpha)>0:
                self.widget.set_alpha (int (alpha))
            else:
                self.widget.set_alpha (65535)

