'''This module implements a hierachy of configuration entries.

The following types of entries are provided:
GroupEntry (type="group")
BoolEntry (type="bool")
StringEntry (type="string")
ListEntry (type="list")
ListStringEntry (type="liststring")
NumericEntry (type="numeric")
'''

import math

import gtk


class Entry:
    '''This is the entry base class. Concret entries must subclass from it.'''
    
    def __init__ (self):
        self.widget = None
        
    def set_default (self, text):
        '''Default values are ignored by default.'''
        pass
        
    def set_title (self, text):
        '''Titles are ignored by default.'''
        pass
        
    def set_tooltip (self, text):
        '''Tooltips are ignored by default.'''
        pass

class ContainerEntry (Entry):
    '''This is an intermediate base class for entries, that can have sub-entries.'''
    
    def __init__ (self):
        Entry.__init__ (self)
        self.sub = []
        
    def set_config (self, config):
        '''Read config for this entry from config map.'''
        for entry in self.sub:
            entry.set_config (config)
        
    def get_config (self, config):
        '''Store config for this entry into config map.'''
        for entry in self.sub:
            entry.get_config (config)
            
    def use_default (self):
        '''Set entry to default value.'''
        for entry in self.sub:
            entry.use_default()


class GroupEntry (ContainerEntry):
    '''This entry represents a grouping without providing settings data itself.
    
    This entry has the following elements:
    title (optional)
    entry (optional, any number)
    '''
    
    def __init__ (self):
        ContainerEntry.__init__ (self)
        self.title = None
        self.widget = None

    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:
            self.widget = gtk.Frame (self.title)
            self.widget.set_border_width (4)
            if len (self.sub) !=0:
                self.table = gtk.Table (len (self.sub), 2)
                self.table.set_border_width (4)                
                self.widget.add (self.table)
                index = 0
                for e in self.sub:
                    (label, widget) = e.get_widget()
                    if widget is not None:
                        if label is None:
                            self.table.attach (widget, 0, 2, index, index+1,
                                yoptions=gtk.SHRINK)
                        else:
                            self.table.attach (label, 0, 1, index, index+1, 
                                xoptions=gtk.SHRINK|gtk.FILL, yoptions=gtk.SHRINK,
                                xpadding = 8)
                            self.table.attach (widget, 1, 2, index, index+1, 
                                yoptions=gtk.SHRINK)
                    index = index + 1                
        return (None, self.widget)

    def set_title (self, text):
        '''Set title (i.e. the frame label).'''
        self.title = text


class BoolEntry (Entry):
    '''Boolean configuration entry, represented as a checkbutton.
        
    This entry has the following elements:
    title
    hint (optional)
    value
    '''
    
    def __init__ (self, id):
        Entry.__init__ (self)
        self.title = ''
        self.id = id
        self.state = False
        self.default = False
        self.tooltip = None
        self.widget = None
    
    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:
            self.widget = gtk.CheckButton (self.title)
            self.widget.set_active (self.state)
            if self.tooltip is not None:
                self.widget.set_tooltip_text (self.tooltip)            
        return (None, self.widget)

    def set_config (self, config):
        '''Read config for this entry from config map.'''
        state = config[self.id]
        if state=='true':
            self.state = True
        else:
            self.state = False
        if self.widget is not None:
            self.widget.set_active (self.state)
            
    def get_config (self, config):
        '''Store config for this entry into config map.'''
        if self.widget is not None:
            self.state = self.widget.get_active()        
        if self.state:
            config[self.id] = 'true'
        else:
            config[self.id] = 'false'

    def set_default (self, text):
        if text=='true':
            self.default = True;
        else:
            self.default = False;
        
    def use_default (self):
        '''Set entry to default value.'''
        self.state = self.default
        self.widget.set_active (self.state)

    def set_title (self, text):
        '''Set title (i.e. the button label).'''
        self.title = text

    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text
        if self.widget is not None:
            self.widget.set_tooltip_text (self.tooltip)

            
class StringEntry (Entry):
    '''String configuration entry, represented as an entry widget.
        
    This entry has the following elements:
    title
    hint (optional)
    value
    '''
    
    def __init__ (self, id):
        Entry.__init__ (self)
        self.title = ''
        self.id = id
        self.state = ''
        self.default = ''
        self.tooltip = None
        self.widget = None
        self.label = None
    
    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:
            self.widget = gtk.Entry()
            self.widget.set_text (self.state)
            self.widget.set_activates_default (True)
            self.label = gtk.Label (self.title)
            self.label.set_alignment (0.0, 0.5)
            if self.tooltip is not None:
                self.widget.set_tooltip_text (self.tooltip)
                self.label.set_tooltip_text (self.tooltip)
        return (self.label, self.widget)

    def set_config (self, config):
        '''Read config for this entry from config map.'''
        self.state = config[self.id]
        if self.widget is not None:
            self.widget.set_text (self.state)
            
    def get_config (self, config):
        '''Store config for this entry into config map.'''
        if self.widget is not None:
            self.state = self.widget.get_text()    
        config[self.id] = self.state    

    def set_default (self, text):
        '''Set the default value for this entry.'''
        self.default = text
        
    def use_default (self):
        '''Set entry to default value.'''
        self.state = self.default
        self.widget.set_text (self.state)

    def set_title (self, text):
        '''Set title (i.e. the label).'''
        self.title = text
        if self.label is not None:
            self.label.set_text (self.title)

    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text


class ListEntry (ContainerEntry):
    '''This entry represents a list of boolean options of which exactly one is true.
    
    This entry has the following elements:
    title
    entry (of type liststring, any number >0)
    hint (optional)
    value
    '''
    
    def __init__ (self, id, limit):
        ContainerEntry.__init__ (self)
        self.title = None
        self.id = id
        self.limit = limit
        self.widget = None
        self.label = None
        self.store = None
        self.state = ''
        self.default = ''
        self.radiomode = False
        self.tooltip = None

    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:
            size = len (self.sub)
            if size<=self.limit:
                self.radiomode = True
            else: # combobox entries don't work with tooltips -> use radiobuttons
                entries = [e for e in self.sub if e.tooltip is not None]
                if len (entries) > 0:
                    print entries
                    self.radiomode = True
            if self.radiomode:
                if self.title is not None:
                    size = size + 1
                if size == 0:
                    size = 1
                self.widget = gtk.Table (size, 1)
                self.widget.set_border_width (6)
                index = 0
                if self.title is not None:
                    widget = gtk.Label (self.title)
                    if self.tooltip is not None:
                        widget.set_tooltip_text (self.tooltip)
                    widget.set_alignment (0.0, 0.5)                    
                    self.widget.attach (widget, 0, 1, index, index+1,      
                        yoptions=gtk.SHRINK, ypadding=4)
                    index = index + 1
                first = None
                for e in self.sub:
                    button = e.get_widget()[1]
                    if first is None:
                        first = button
                    else:
                        button.set_group (first)
                    self.widget.attach (button, 0, 1, index, index+1,      
                        yoptions=gtk.SHRINK)
                    index = index + 1                    
            else:
                self.store = gtk.ListStore (str, str)
                self.widget = gtk.ComboBox (self.store)
                if self.tooltip is not None:
                    self.widget.set_tooltip_text (self.tooltip)
                cell = gtk.CellRendererText()
                self.widget.pack_start (cell, True)
                self.widget.add_attribute (cell, 'text', 0)
                for e in self.sub:
                    if issubclass (e.__class__, ListStringEntry):
                        iter = self.store.append()
                        self.store.set (iter, 0, e.title)
                        self.store.set (iter, 1, e.handle)
                if self.title is not None:
                    self.label = gtk.Label (self.title)
                    self.label.set_alignment (0.0, 0.5)
                    if self.tooltip is not None:
                        self.label.set_tooltip_text (self.tooltip)                    
            self._set_config()
        return (self.label, self.widget)
            
    def _set_config (self):
        '''Adjust widget.'''
        if self.widget is not None:
            if self.radiomode:
                active = [e for e in self.sub if e.handle==self.state]
                if len (active) > 0:
                    active[0].get_widget()[1].set_active (True)
            else:
                rows = [r for r in self.store if r[1]==self.state]
                if len (rows) > 0:
                    self.widget.set_active_iter (rows[0].iter)
            
    def set_config (self, config):
        '''Read config for this entry from config map.'''
        self.state = config[self.id]
        self._set_config()
        
    def get_config (self, config):
        '''Store config for this entry into config map.'''
        if self.radiomode:
            active = [e for e in self.sub if e.get_widget()[1].get_active()]
            if len (active) == 0:
                self.state = ''
            else:
                self.state = active[0].handle
        elif self.widget is not None:
            active = self.widget.get_active_iter()
            if active is None:
                self.state = ''
            else:
                self.state = self.store.get (active, 1)[0]
        config[self.id] = self.state    
            
    def set_default (self, text):
        '''Set the default value for this entry.'''
        self.default = text
                    
    def use_default (self):
        '''Set entry to default value.'''
        self.state = self.default
        self._set_config()

    def set_title (self, text):
        '''Set title (i.e. the label).'''
        self.title = text
        if self.label is not None:
            self.label.set_text (self.title)
            
    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text
            

class ListStringEntry (Entry):
    '''Entry within a list entry.
        
    This entry has the following elements:
    title
    handle    
    hint (optional)
    '''
    
    def __init__ (self):
        Entry.__init__ (self)
        self.title = ''
        self.handle = None
        self.widget = None
        self.tooltip = None

    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''        
        if self.widget is None:
            self.widget = gtk.RadioButton (label = self.title)
            if self.tooltip is not None:
                self.widget.set_tooltip_text (self.tooltip)
        return (None, self.widget)

    def set_config (self, config):
        '''Read config for this entry from config map.'''
        pass # no data
        
    def get_config (self, config):
        '''Store config for this entry into config map.'''
        pass # no data
            
    def use_default (self):
        '''Set entry to default value.'''
        pass # no data
        
    def set_title (self, text):
        '''Set title (i.e. the label).'''
        self.title = text

    def set_handle (self, text):
        '''Set handle by which this string can be identified.'''
        self.handle = text
        
    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text
        

class NumericEntry (Entry):
    '''Numeric configuration entry, represented as a hscale or spinbutton widget.
        
    This entry has the following elements:
    title
    hint (optional)
    value
    min
    max
    digits (optional)
    '''
    
    def __init__ (self, id, exact):
        Entry.__init__ (self)
        self.title = ''
        self.id = id
        self.state = ''
        self.default = '0'
        self.tooltip = None
        self.widget = None
        self.label = None
        self.exact = exact
        self.min = 0
        self.max = 0
        self.digits = 0
    
    def get_widget (self):
        '''Return widget-pair, that represents this entry.
        
        The first widget is used as a label and can be None. The second widget contains
        the actual data.'''
        if self.widget is None:
            step = 1
            if math.fabs (self.min)<5 and math.fabs (self.max)<5:
                step = math.pow (10, -self.digits)
            page = 10 * step
            if page>=self.max-self.min:
                page = step
            adjustment = gtk.Adjustment (float (self.state), self.min, self.max, step,
                page)
            if self.exact:
                self.widget = gtk.SpinButton (adjustment, digits=self.digits)
                self.widget.set_numeric (True)
                self.widget.set_activates_default (True)
            else:
                self.widget = gtk.HScale (adjustment)
                self.widget.set_digits (self.digits)
            self.label = gtk.Label (self.title)
            self.label.set_alignment (0.0, 0.5)
            if self.tooltip is not None:
                self.widget.set_tooltip_text (self.tooltip)
                self.label.set_tooltip_text (self.tooltip)
        return (self.label, self.widget)

    def set_config (self, config):
        '''Read config for this entry from config map.'''
        self.state = config[self.id]
        if self.widget is not None:
            self.widget.set_value (self.state)
            
    def get_config (self, config):
        '''Store config for this entry into config map.'''
        if self.widget is not None:
            self.state = str (self.widget.get_value())
        config[self.id] = self.state    

    def set_default (self, text):
        '''Set the default value for this entry.'''
        self.default = text
        
    def use_default (self):
        '''Set entry to default value.'''
        self.state = self.default
        self.widget.set_value (float (self.state))

    def set_title (self, text):
        '''Set title (i.e. the label).'''
        self.title = text
        if self.label is not None:
            self.label.set_text (self.title)

    def set_tooltip (self, text):
        '''Set tooltip for this entry.'''
        self.tooltip = text
        
    def set_min (self, value):
        '''Set minimum value.'''
        self.min = value
    
    def set_max (self, value):
        '''Set maximum value.'''
        self.max = value

    def set_digits (self, value):
        '''Set number of decimal digits.'''
        self.digits = value

