"""This module implements a configuration section, which is handled as a tab under
FinalConfig.

"""

import gtk


class Section:
    '''Section, implemented as node in a tree.'''
    
    def __init__ (self, parent_section=None, hidden=False):
        '''Construct a new section.
        '''
        self.title = '<no title>'
        self.parent_section = parent_section
        self.rowref = None
        self.hidden = hidden
        self.sub = [] # entries
        self.widget = None

    def get_widget (self):
        '''Return widget, that represents this section.'''
        if self.widget is None:
            size = len (self.sub)
            if size==0:
                size = 1
            self.widget = gtk.Table (size, 2)
            self.widget.set_border_width (4)
            index = 0
            for e in self.sub:
                (label, widget) = e.get_widget()
                if widget is not None:
                    if label is None:
                        self.widget.attach (widget, 0, 2, index, index+1,
                            yoptions=gtk.SHRINK)
                    else:
                        self.widget.attach (label, 0, 1, index, index+1, 
                            xoptions=gtk.SHRINK|gtk.FILL, yoptions=gtk.SHRINK,
                            xpadding = 8)
                        self.widget.attach (widget, 1, 2, index, index+1, 
                            yoptions=gtk.SHRINK)                
                index = index + 1
        return self.widget

    def set_config (self, config):
        '''Read config for all entries in this section from config map.'''
        for entry in self.sub:
            entry.set_config (config)

    def get_config (self, config):
        '''Store config for all entries in this section into config map.'''
        for entry in self.sub:
            entry.get_config (config)

    def use_default (self):
        '''Use default values.'''
        for entry in self.sub:
            entry.use_default()
            
    def set_title (self, text):
        '''Set title (i.e. the line used in the side pane.'''
        self.title = text

class SectionList:
    '''This class implements the section list pane via a gtk tree view.'''
    
    def __init__ (self):
        self.main = gtk.ScrolledWindow()
        self.main.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.model = gtk.TreeStore (str, object)
        self.view = gtk.TreeView (self.model)
        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn ('', cell)
        self.view.append_column (column)
        column.add_attribute (cell, 'text', 0)
        self.view.set_headers_visible (False)
        self.view.set_rules_hint (False)
        self.view.set_reorderable (False)        
        self.view.set_enable_search (True)
        self.view.set_search_column (0)
        self.main.add (self.view)
        
    def add (self, section):
        '''Add a new scetion to the list.
        
        This functions sets section.rowref to the row used to display the section in the
        tree view.
        '''
        row = None
        parent = section.parent_section
        if parent is not None and parent.rowref is not None:
            row = self.model.get_iter (section.parent_section.rowref.get_path())  
        iter = self.model.append (row)
        self.model.set (iter, 0, section.title)
        self.model.set (iter, 1, section)
        
        section.rowref = gtk.TreeRowReference (self.model, self.model.get_path (iter)) 
        
    def get_selected_section (self):
        '''Return the currently selected section.'''
        selection = self.view.get_selection()
        pathlist = selection.get_selected_rows()[1]
        if len (pathlist)==0:
            return None
        return self.model.get_value (self.model.get_iter (pathlist[0]), 1)
        
    def select (self, section):
        '''Select section in tree view.'''
        iter = self.model.iter_children (None)
        while iter:
            if self.model.get (iter, 1)==section:
                selection = self.view.get_selection()
                selection.select_iter (iter)
                break                
            else:
                iter = self.model.iter_next (iter)

    def set_config (self, config):
        '''Read config for all entries from config map.'''
        for row in self.model:
            row[1].set_config (config)

    def get_config (self, config):
        '''Store config for all entries into config map.'''
        for row in self.model:
            row[1].get_config (config)

    def use_default (self):
        '''Use default values'''
        for row in self.model:
            row[1].use_default()
            
    def update_titles (self, parent = None):
        '''Copy titles from section into titles column.'''
        if parent is None:
            parent = self.model
        for row in parent:
            row[0] = row[1].title
            self.update_titles (row.iterchildren())

