'''This module implements a finalconfig session (i.e. a dialogue for a single client's
configuration.

'''

import xml.dom.minidom

import gtk
import dbus
import dbus.service

import finalconfig.section
import finalconfig.desc
import finalconfig.config


class Session (dbus.service.Object):
    '''This class implements a finalconfig session (i.e. a dialogue for a single client's
    configuration).
    '''
    
    def __init__ (self, manager, config_path, desc_path, session_handle, app_name,
        config):
        '''manager must implement a close function:
        close (session)
        changed (session)
        shutdown()
        '''
        name = dbus.service.BusName ('de.zpages.FinalConfig',
            dbus.service.SessionBus())
        dbus.service.Object.__init__ (self, conn = dbus.SessionBus(),
            object_path = session_handle, bus_name = name)        
        self.manager = manager
        self.config_path = config_path
        self.desc_path = desc_path
        self.app_name = app_name
        self.uimanager = gtk.UIManager()
        self.actiongroup = gtk.ActionGroup ('main')
        self.uimanager.insert_action_group (self.actiongroup)
        self._create_actions (config)
        self._create_ui_elements (config)
        self._create_window (app_name, config)
        self.section = None
        self.sections.view.connect ('cursor-changed', self._selection_changed)
        handler = finalconfig.desc.ContentHandler (self, config)
        xml.sax.parse (desc_path, handler)
        self.config = finalconfig.config.read_config (self.config_path)
        self.sections.set_config (self.config)
        self.sections.update_titles()

    def _create_actions (self, config):
        action = gtk.Action ('app', 'App', None, None)
        self.actiongroup.add_action (action)
        action = gtk.Action ('quit', None, None, gtk.STOCK_QUIT)
        action.connect ("activate", self._quit)
        self.actiongroup.add_action (action)

    def _create_ui_elements (self, config):
        merge_id = self.uimanager.new_merge_id()
        self.uimanager.add_ui (merge_id, '/', 'MenuBar', '', gtk.UI_MANAGER_MENUBAR,
            False)
        self.uimanager.add_ui (merge_id, '/MenuBar', 'App', 'app', gtk.UI_MANAGER_MENU,
            False)
        self.uimanager.add_ui (merge_id, '/MenuBar/App', 'Quit', 'quit',
            gtk.UI_MANAGER_MENUITEM, False)
            
    def _create_window (self, app_name, config):
        self.window = gtk.Window (gtk.WINDOW_TOPLEVEL)
        self.window.set_default_size (int (float (config['window.width'])),
            int (float (config['window.height'])))
        self.window.set_size_request (320, 200)
        self.window.set_title ("Preferences: " + app_name)
        self.top = gtk.VBox()
        self.window.add (self.top)
        self.paned1 = gtk.HPaned()
        self.has_main_menu = config['window.menu']=='true'
        if self.has_main_menu:
            self.top.pack_start (self.uimanager.get_widget ('/MenuBar'), False, False)
        self.top.pack_end (self.paned1)
        self.sections = finalconfig.section.SectionList()
        self.paned1.add1 (self.sections.main)
        self.main = gtk.VBox (spacing = 8)
        self.paned1.add2 (self.main)
        self.paned1.set_position (160)
        self._create_buttons()
        self.content2 = gtk.ScrolledWindow()
        self.content2.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.main.pack_start (self.content2, True, True)
        self.content = gtk.Viewport()
        self.content.set_shadow_type (gtk.SHADOW_NONE)
        self.content2.add (self.content)
        self.window.connect ("destroy", self._destroy) 
        self.window.add_accel_group (self.uimanager.get_accel_group())

    def _create_buttons (self):
        self.buttons = gtk.HButtonBox()
        self.buttons.set_layout (gtk.BUTTONBOX_END)
        self.buttons.set_spacing (4)
        self.main.pack_end (self.buttons, False, False)
        self.apply = gtk.Button (stock=gtk.STOCK_APPLY)
        self.apply.connect ('clicked', self._apply)
        self.buttons.add (self.apply)
        self.default = gtk.Button (label='Default')
        self.default.connect ('clicked', self._default)
        self.buttons.add (self.default)
        self.cancel = gtk.Button (stock=gtk.STOCK_CANCEL)
        self.cancel.connect ('clicked', self._destroy)
        self.buttons.add (self.cancel)
        self.okay = gtk.Button (stock=gtk.STOCK_OK)
        self.okay.connect ('clicked', self._okay)
        self.buttons.add (self.okay)
        self.okay.set_flags (gtk.CAN_DEFAULT)
        self.okay.grab_default() 
        
    @dbus.service.signal (dbus_interface='de.zpages.FinalConfig', signature='s')
    def reply (self, config_path):
        pass
            
    @dbus.service.signal (dbus_interface='de.zpages.FinalConfig', signature='')
    def close_session (self):
        pass
        
    @dbus.service.method (dbus_interface='de.zpages.FinalConfig', in_signature='',
        out_signature='')
    def abort_session (self):
        '''Client notifies FinalConfig, that it is no longer interested in the
        results from the preferences dialogue.'''
        self.window.hide()
        self.remove_from_connection()
        self.manager.close (self)
        
    @dbus.service.method (dbus_interface='de.zpages.FinalConfig', in_signature='',
        out_signature='')
    def highlight (self):
        '''Bring the session window to the user's attention.'''
        self.window.show()
        self.window.present()
        
    def _destroy (self, widget, data=None):
        self.window.hide()
        self.close_session()
        self.remove_from_connection()
        self.manager.close (self)
        
    def _okay (self, widget):
        self._apply (widget)
        self.window.hide()
        self.close_session()
        self.remove_from_connection()
        self.manager.close (self)
        
    def _default (self, widget):
        self.sections.use_default()
        self._okay (widget)
        
    def _apply (self, widget):
        self.config = dict()
        self.sections.get_config (self.config)
        self._write_config (self.config, self.config_path)
        self.reply (self.config_path)
        self.manager.changed (self)
        
    def _write_config (self, config, path):
        finalconfig.config.write_config (config, path)

    def add_section (self, section):
        '''Add a new section to the window.      
        '''
        self.sections.add (section)        

    def show (self):
        '''Show window and make sure a section is selected.'''
        if self.section is None and self.sections.model.get_iter_first() is not None:
            section = self.sections.model.get_value (self.sections.model.get_iter_first(), 1)
            self.select (section)
        self.window.show_all()      
        
    def select (self, section):
        '''Select a section.'''
        if self.section is not None:
            self.content.remove (self.section.get_widget())
            self.section = None
        self.sections.select (section)
        widget = section.get_widget()
        self.content.add (widget)
        widget.show_all()
        self.section = section

    def _selection_changed (self, widget):
        section = self.sections.get_selected_section()
        if section is not None and section!=self.section:
            self.select (section)

    def _quit (self, widget):
        self.manager.shutdown()
        
    def on_config_changed (self, config):
        menu = config.config['window.menu']=='true'
        if menu != self.has_main_menu:
            self.has_main_menu = menu
            if self.has_main_menu:
                self.top.pack_start (self.uimanager.get_widget ('/MenuBar'), False, False)
            else:
                self.top.remove (self.uimanager.get_widget ('/MenuBar'))

